package com.shanghaichengdi.downloadcenterserver.controller;

import com.alibaba.fastjson.JSONObject;
import com.shanghaichengdi.downloadcentercommon.util.ImageDownload;
import com.shanghaichengdi.downloadcentercommon.util.SSLUtil;
import com.shanghaichengdi.downloadcenterserver.cache.UrlData;
import com.shanghaichengdi.downloadcenterserver.domain.PicDownload;
import com.shanghaichengdi.downloadcenterserver.faced.PicDownloadFaced;
import com.shanghaichengdi.downloadcenterserver.faced.PicDownloadHistoryFaced;
import com.shanghaichengdi.downloadcenterserver.util.PrintLogUtils;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import java.io.File;
import java.net.URL;
import java.net.URLDecoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

/**
 * @author gengaoliu
 */
@Slf4j
@CrossOrigin
@RestController
@RequestMapping(value = "/download")
public class PicDownloadController {
  @PostMapping("/pictures")
  public String downloadPictures(@RequestBody String data) {
    try {
      PicDownload picDownload = JSONObject.parseObject(data, PicDownload.class);
      log.info("接收到的数据为：{}", JSONObject.toJSON(picDownload));
      PicDownloadFaced faced =
              new PicDownloadFaced(picDownload.getImageUrls(), picDownload.getPassTime(), picDownload.getId(), picDownload.getBayonetId());
      try {
        faced.start();
      } catch (Exception e) {
        PrintLogUtils.printErrorLogInfo(e, "下载失败,原因：");
        return new ResultMsg<String>().setMsg("下载失败").setCode(ResultState.ERROR).setData(null).toJSON();
      }
      log.info("下载成功");
      return new ResultMsg<String>().setMsg("下载成功").setCode(ResultState.SUCCESS).setData(null).toJSON();
    }catch (Exception e){
      PrintLogUtils.printErrorLogInfo(e,"pictures下载失败，数据为："+data+";");
      return new ResultMsg<String>().setMsg("下载失败").setCode(ResultState.ERROR).setData(null).toJSON();
    }
  }


  @PostMapping("/pictures/history")
  public String downloadPicturesHistory(@RequestBody String data) {
    PicDownload picDownload = JSONObject.parseObject(data, PicDownload.class);
//    log.info("接收到的数据为：{}",JSONObject.toJSON(picDownload));
    PicDownloadHistoryFaced faced =
        new PicDownloadHistoryFaced(picDownload.getImageUrls(), picDownload.getPassTime(),picDownload.getId(),picDownload.getBayonetId());
    try {
      if (!"8".equals(faced.getBayonetId())) {
        faced.start();
      }
    } catch (Exception e) {
//      log.info("下载失败,原因：{}",e.getMessage());
      return new ResultMsg<String>().setMsg("下载失败").setCode(ResultState.ERROR).setData(null).toJSON();
    }
//    log.info("下载成功");
    return new ResultMsg<String>().setMsg("下载成功").setCode(ResultState.SUCCESS).setData(null).toJSON();
  }

  //需要重新弄一个接口用于下载识别结果图,不要和上面的下载混在一起,上面的只用于下载原图片以及数据回调

  //TODO 发布时修改
  private static final String ANALYSED_PICTURE_PATH = "/data/dataShareFolder/bayonetPhoto/analysedPicture/";
//  private static final String ANALYSED_PICTURE_PATH = "E:/disk/dataShareFolder/bayonetPhoto/analysedPicture/";

  @PostMapping("/downloadAnalysedPicture")
  public String downloadAnalysedPicture(@RequestBody Map<String,String> map) {
    String pictureUrl = map.get("pictureUrl");
    String bayonetId = map.get("bayonetId");
//    String dirs = pictureUrl.substring(pictureUrl.lastIndexOf("/") + 1, pictureUrl.lastIndexOf("."));
    String[] split = pictureUrl.split("/");
    String fileName = split[split.length - 1];
    String lastLevel = split[split.length - 2];
    String rootLevel = split[split.length - 3];
    new File(ANALYSED_PICTURE_PATH + bayonetId + "/" + rootLevel + "/" + lastLevel).mkdirs();
    //这里图片就不进行重命名了
    String filePath = ANALYSED_PICTURE_PATH + bayonetId + "/" + rootLevel + "/" + lastLevel + "/" + fileName;
      try {
          ImageDownload.download(pictureUrl, filePath);
      } catch (Exception e) {
          log.error("下载异常,对应卡口为{},对应图片路径为{},异常内容为{}",bayonetId,pictureUrl,e.getMessage());
          throw new RuntimeException(e);
      }
      return "success";
  }

  @GetMapping("test")
  public String test() {
    return "test";
  }
  @Deprecated
  @PostMapping("/first/pictures")
  public String downloadFirstPictures(@RequestBody String data) {
    PicDownload picDownload = JSONObject.parseObject(data, PicDownload.class);
    int resultNum = this.downloadPics(picDownload.getImageUrls(), picDownload.getPassTime());
    if (resultNum != picDownload.getImageUrls().size()) {
      return new ResultMsg<String>()
          .setMsg("下载失败")
          .setCode(ResultState.ERROR)
          .setData(String.valueOf(resultNum))
          .toJSON();
    }
    return new ResultMsg<String>()
        .setMsg("下载成功")
        .setCode(ResultState.SUCCESS)
        .setData(String.valueOf(resultNum))
        .toJSON();
  }

  @Deprecated
  private int downloadPics(List<String> imageUrls, String passTime) {
    String time = passTime.split("\\+")[0];
    Date times;
    try {
      SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      times =
          formatter.parse(
              time.split("\\.")[0].split("T")[0] + " " + time.split("\\.")[0].split("T")[1]);
    } catch (ParseException e) {
      e.printStackTrace();
      return -1;
    }
    SimpleDateFormat formatter = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
    SimpleDateFormat formatter1 = new SimpleDateFormat("yyyyMMdd_HHmmss");
    String dir = formatter.format(times);
    String newTime = formatter1.format(times) + time.split("\\.")[1];
    String year = dir.split("\\_")[0];
    String month = dir.split("\\_")[1];
    String day = dir.split("\\_")[2];
    String hour = dir.split("\\_")[3];
    String dirs = year + "\\" + year + month + day + "\\" + hour;

    int i = 0;
    File photoDir = new File("\\\\K6XL82KY31YIWFC\\photo\\" + dirs);
    photoDir.mkdirs();
    for (String imageUrl : imageUrls) {
      boolean isSuccess = false;
      URL url = null;
      try {
        url = new URL(URLDecoder.decode(imageUrl, "utf-8"));
        if ("https".equalsIgnoreCase(url.getProtocol())) {
          SSLUtil.ignoreSsl();
        }
        String photoFile =
            "\\\\K6XL82KY31YIWFC\\photo\\" + dirs + "\\" + "P" + i + "_" + newTime + ".jpg";
        System.out.println(url);
        isSuccess = ImageDownload.download(imageUrl, photoFile);
        if (!isSuccess) {
          return i;
        }
        i++;
      } catch (Exception e) {
        e.printStackTrace();
        return i;
      }
    }
    return i;
  }
}
